package ctbrec.recorder;

import java.time.Duration;
import java.time.Instant;
import java.util.concurrent.ThreadPoolExecutor;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class ThreadPoolScaler {
    private static final Logger LOG = LoggerFactory.getLogger(ThreadPoolScaler.class);

    private ThreadPoolExecutor threadPool;
    private int configuredPoolSize;

    private int[] values = new int[20];
    private int index = -1;
    private Instant lastAdjustment = Instant.EPOCH;


    public ThreadPoolScaler(ThreadPoolExecutor threadPool, int configuredPoolSize) {
        this.threadPool = threadPool;
        this.configuredPoolSize = configuredPoolSize;
    }

    public void tick() {
        values[getNextIndex()] = threadPool.getActiveCount();
        int sum = 0;
        for (int i = 0; i < values.length; i++) {
            sum += values[i];
        }
        double average = sum / (double) values.length;

        if (Duration.between(lastAdjustment, Instant.now()).toMillis() > 2000) {
            int coreSize = threadPool.getCorePoolSize();
            if (average > 0.75 * coreSize) {
                threadPool.setCorePoolSize(coreSize + 1);
                lastAdjustment = Instant.now();
            } else if (average > 0.25 * coreSize) {
                threadPool.setCorePoolSize(Math.max(configuredPoolSize, coreSize - 1));
                lastAdjustment = Instant.now();
            }
            LOG.trace("Adjusted scheduler pool size to {}", threadPool.getCorePoolSize());
        }
    }

    private synchronized int getNextIndex() {
        index++;
        if (index == values.length) {
            index = 0;
        }
        return index;
    }
}
